@@ -5,7 +5,7 @@
 diffnb <- function(x, y, verbose=NULL) {
 	if (!inherits(x, "nb")) stop("not a neighbours list")
 	if (!inherits(y, "nb")) stop("not a neighbours list")
-        if (is.null(verbose)) verbose <- get("verbose", env = .spdepOptions)
+        if (is.null(verbose)) verbose <- get.VerboseOption()
         stopifnot(is.logical(verbose))
 	n <- length(x)
 	if (n < 1) stop("non-positive length of x")
@@ -31,6 +31,12 @@ diffnb <- function(x, y, verbose=NULL) {
 	attr(res, "region.id") <- attr(x, "region.id")
 	attr(res, "call") <- match.call()
 	res <- sym.attr.nb(res)
+        NE <- n + sum(card(res))
+        if (get.SubgraphOption() && get.SubgraphCeiling() > NE) {
+          ncomp <- n.comp.nb(res)
+          attr(res, "ncomp") <- ncomp
+          if (ncomp$nc > 1) warning("neighbour object has ", ncomp$nc, " sub-graphs")
+        }
 	res
 }	
 	
