@@ -5,7 +5,7 @@
 nb2mat <- function(neighbours, glist=NULL, style="W", zero.policy=NULL)
 {
         if (is.null(zero.policy))
-            zero.policy <- get("zeroPolicy", envir = .spdepOptions)
+            zero.policy <- get.ZeroPolicyOption()
         stopifnot(is.logical(zero.policy))
 	if(!inherits(neighbours, "nb")) stop("Not a neighbours list")
 	listw <- nb2listw(neighbours, glist=glist, style=style,
@@ -31,7 +31,7 @@ listw2mat <- function(listw) {
 }
 
 
-mat2listw <- function(x, row.names=NULL, style=NULL) {
+mat2listw <- function(x, row.names=NULL, style=NULL, zero.policy=NULL) {
 	if (!(is.matrix(x) || is(x, "sparseMatrix"))) stop("x is not a matrix")
 	n <- nrow(x)
 	if (n < 1) stop("non-positive number of entities")
@@ -39,6 +39,8 @@ mat2listw <- function(x, row.names=NULL, style=NULL) {
 	if (n != m) stop("x must be a square matrix")
 	if (any(x < 0)) stop("values in x cannot be negative")
 	if (any(is.na(x))) stop("NA values in x not allowed")
+        if (is.null(zero.policy))
+            zero.policy <- get.ZeroPolicyOption()
     	if (!is.null(row.names)) {
 		if(length(row.names) != n)
             		stop("row.names wrong length")
@@ -70,7 +72,8 @@ mat2listw <- function(x, row.names=NULL, style=NULL) {
             class(df) <- c(class(df), "spatial.neighbour")
             attr(df, "region.id") <- row.names
             attr(df, "n") <- dim(xC)[1]
-            res <- sn2listw(df)
+            res <- sn2listw(df, style=style, zero.policy=zero.policy,
+                from_mat2listw=TRUE)
             neighbours <- res$neighbours
             weights <- res$weights
         } else {
@@ -92,15 +95,30 @@ mat2listw <- function(x, row.names=NULL, style=NULL) {
  	attr(neighbours, "call") <- NA
         attr(neighbours, "sym") <- is.symmetric.nb(neighbours, 
 		verbose=FALSE, force=TRUE)
+        cnb <- card(neighbours)
+        if (any(cnb == 0L)) {
+            if (!zero.policy) {
+                warning("no-neighbour observations found, set zero.policy to TRUE;\nthis warning will soon become an error")
+            }
+        }
+
+        NE <- length(neighbours) + sum(cnb)
+        if (get.SubgraphOption() && get.SubgraphCeiling() > NE) {
+          ncomp <- n.comp.nb(neighbours)
+          attr(neighbours, "ncomp") <- ncomp
+          if (ncomp$nc > 1) warning("neighbour object has ", ncomp$nc, " sub-graphs")
+        }
+
 	res <- list(style=style, neighbours=neighbours, weights=weights)
 	class(res) <- c("listw", "nb")
 	attr(res, "region.id") <- attr(neighbours, "region.id")
 	attr(res, "call") <- match.call()
+        attr(res, "zero.policy") <- zero.policy
         if (style != "M") {
 	    if (!(style %in% c("W", "B", "C", "S", "U", "minmax")))
 		stop(paste("Style", style, "invalid"))
             res <- nb2listw(res$neighbours, glist=res$weights, style=style,
-                zero.policy=TRUE)
+                zero.policy=zero.policy)
         }
 	res
 }
