@@ -1,7 +1,7 @@
-/* Copyright 2001 by Nicholas Lewin-Koh. */
+/* Copyright 2001 by Nicholas Lewin-Koh.
+ * interrupt added RSB 2024 */
 
-#include<R.h>
-#include<Rinternals.h>
+#include "spdep.h"
 
 #define BLACK 1
 #define WHITE 0
@@ -34,6 +34,7 @@ SEXP g_components(SEXP nblst, SEXP cmpnm){
   }
 
   for(i=0; i < nvert; i++){
+    R_CheckUserInterrupt();
     if(INTEGER(visited)[i]==WHITE){ 
       INTEGER(visited)[i]=BLACK;
       if(INTEGER(VECTOR_ELT(nblst,i))[0]==0){
